/*
 * Capacitive Touch Skin
 ---
 * @author Sjoerd Legué
 * @email sjoerdlegue@gmail.com
 
 * Demo code initially written by Limor Fried/Ladyada for Adafruit Industries.
 * Designed specifically to work with the MPR121 Breakout.
 */

#include <Wire.h>
#include "Adafruit_MPR121.h"

Adafruit_MPR121 cap = Adafruit_MPR121();

// Config variables for treshold, etc;
boolean DEBUG = false;
boolean ADVANCED_MODE = true;
int t_treshold = 800; // Time per zone touch limit (over this ms. value is direction multip. 1)
int e_treshold = 10; // Empty zone count limit (when detected that current 'touched' is empty)
int c_limit = 10; // Processing connection attempt limit
int c_interval = 500; // Connection attempt interval

// Variable inits;
uint16_t previous = 0;
uint16_t current = 0;
int e_count = 0;
int c_count = 0;
int timearray[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};

// Advanced mode variables;
int am_c_loops = 30;
int am_c_array[12] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
int am_touch_treshold = 4;
boolean am_touch_previous[12] = {false, false, false, false, false, false, false, false, false, false, false, false};
boolean am_touch_current[12] = {false, false, false, false, false, false, false, false, false, false, false, false};

//double directionX[12] = {0, -1, 0, 1, -1, 0, 1, -1, 0, 1, 0, 0};
//double directionY[12] = {0, -1, -1, -1, 0, 0, 0, 1, 1, 1, 0, 0};

void setup() {
  // Start serial communication with IDE and/or Processing Script;
  Serial.begin(9600);
  while(!Serial) {
    delay(10);
  }

  // Initialising sequence for the MPR121 breakout;
  (DEBUG ? Serial.println("Initialising MPR121 sensor...") : true);
  if(!cap.begin(0x5A)) {
    if(cap.begin(0x5B)) {
      (DEBUG ? Serial.println("NOTE: Script is written and tested to work with 5V, please change supplied voltage") : true);
    }
    else {
      (DEBUG ? Serial.println("ERROR: MPR121 not found, check wiring? (SCL-A5, SDA-A4)") : true);
      while (1);
    }
  }

  // Establishing connection with Processing through serial;
  (DEBUG ? Serial.println("Start connecting with Processing...") : true);
  while(Serial.available() <= 0) {
    if(c_count == c_limit) {
      (DEBUG ? Serial.println("ERROR: Serial connection attemps failed!") : true);
      Serial.println("SE");
      break;
    }
    c_count++;
    Serial.println("A");

    // Delay connection;
    delay(c_interval);
  }
  if(Serial.available() > 0) {
    (DEBUG ? Serial.println("Serial connection established!") : true);
    Serial.println("SR");

    // Sending set-up data;
    Serial.print("S:t_treshold="); Serial.println(t_treshold);
    Serial.print("S:e_treshold="); Serial.println(e_treshold);
  }
  
  // Calibrating sensors;
  if(ADVANCED_MODE) {
    (DEBUG ? Serial.println("Starting calibrating sequence...") : true);
    
    // Loop measuring zones, then lock baseline values after a determined amount of loops;
    for (uint8_t i=0; i < am_c_loops; i++) {
      
      // Loop for each port;
      for (uint8_t j=0; j<12; j++) {
        current = cap.touched();

        // Initial loop, set first value;
        if(i == 0) {
          am_c_array[j] = cap.filteredData(j);
          delay(50);
        }
        else if(i < 12) {
          am_c_array[j] = am_c_array[j] + cap.filteredData(j);
          delay(50);
        }
        else if(i == 12) {
          am_c_array[j] = am_c_array[j] / 12;
          (DEBUG ? Serial.print("[") : true); (DEBUG ? Serial.print(j) : true); (DEBUG ? Serial.print("] baseline: ") : true); (DEBUG ? Serial.println(am_c_array[j]) : true);
        }
      }
    }
  
    (DEBUG ? Serial.println("Calibrating done!") : true);
    Serial.println("CR");
  }
  else {
    Serial.println("CF");
  }
}

void loop() {
  // Get values and check wether state change is detected;
  current = cap.touched();
  for(uint8_t i=0; i<12; i++) {

    // Detect Advanced-mode touch;
    if(ADVANCED_MODE) {
      int diff = am_c_array[i] - cap.filteredData(i);
      if(abs(diff) > am_touch_treshold) {
        am_touch_current[i] = true;
        (DEBUG ? Serial.print("Advanced touch on zone ") : true); (DEBUG ? Serial.println(i) : true);
      }
      else {
        am_touch_current[i] = false;
      }
    }

    // Detect if zone is touched;
    if(((current & _BV(i)) && !(previous & _BV(i))) || (!am_touch_previous[i] && am_touch_current[i])) {
      
      // Start counting the time a zone is touched;
      timearray[i] = millis();
      //(DEBUG ? Serial.println(timearray[i]) : true);

      // Detect if this is new recording;
      if(!previous && current) {
        Serial.println("RN");
      }
    }
    
    // Detect if zone is UNtouched;
    if((!(current & _BV(i)) && (previous & _BV(i))) || (am_touch_previous[i] && !am_touch_current[i])) {
      
      // Calculate the difference between recorded time and now again;
      int t = millis();
      int d = t - timearray[i];
      float m = 0;
      
      // Send multiplication data to Serial;
      if(d > t_treshold) {
        m = 1;
      }
      else {
        //(DEBUG ? Serial.println(d % t_treshold) : true);
        m = (float)d / t_treshold;
      }

      // Return multiplication value to Processing and reset zone value;
      Serial.print(i); Serial.print(":"); Serial.println(m);
      timearray[i] = 0;
    }
  }
  
  // Detect if all zones are empty;
  if(ADVANCED_MODE) {
    boolean am_previous_hasvalue = false;
    boolean am_current_hasvalue = false;
    for(uint8_t i=0; i<12; i++) {
      (am_touch_previous[i] ? am_previous_hasvalue = true : am_previous_hasvalue = false);
      (am_touch_current[i] ? am_current_hasvalue = true : am_current_hasvalue = false);
    }
    
    if(am_previous_hasvalue && !am_current_hasvalue) {
      e_count++;
    }
  }
  
  if((previous != 0 && current == 0) || (e_count != 0)) {
    e_count++;
    if(e_count == e_treshold + 1) {
      Serial.println("RD");
      if(Serial.available() <= 0) {
        (DEBUG ? Serial.println("NOTE: Processing script not connected, check script!") : true);
      }
      e_count = 0;
    }
  }
  
  // Pass states to a cache;
  for(uint8_t i=0; i<12; i++) {
    am_touch_previous[i] = am_touch_current[i];
  }
  previous = current;
  return;
}
